//  fixDVD CLI

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

int usage()
{
    printf("fixDVD version 1.2\n");
    printf("usage: fixDVD pathToDVD [pathToDVD...]\n");
    printf("where pathToDVD is the path to the folder containing VIDEO_TS folder\n");
    printf("or VIDEO_TS folder itself (folder containing VIDEO_TS.IFO and VIDEO_TS.BUP\n");
    return 1;
}

int main (int argc, const char * argv[])
{
    #define EditSize 6
    const unsigned char fixBuffer[][EditSize] = {
        { 0x30, 0x06, 0x00, 0x01, 0x01, 0x83 }, // Jump to title set 1, title 1, root menu
        { 0x30, 0x06, 0x00, 0x01, 0x02, 0x83 }, // Jump to title set 2, title 1, root menu
        { 0x30, 0x02, 0x00, 0x00, 0x00, 0x01 }};// Jump to title 1
    const int NumBuffers = sizeof(fixBuffer) / sizeof(fixBuffer[0]);
    unsigned char bupBuffer[EditSize], ifoBuffer[EditSize];
    char ifoFileName[1024], bupFileName[1024];
    struct stat statInfo;
    FILE* tsFile;
    size_t numBytes;
    int file, fb;
    
    if (argc <= 1)
        return usage();
    
    for (file = 1; file < argc; file++)
    {
        strcpy(ifoFileName, argv[file]); // /Users/Shared/Movies/Agora/ VIDEO_TS/VIDEO_TS.IFO
        if (ifoFileName[strlen(ifoFileName)-1] != '/')
            strcat(ifoFileName, "/");
        strcpy(bupFileName, ifoFileName);
        strcat(ifoFileName, "VIDEO_TS/VIDEO_TS.IFO");
        strcat(bupFileName, "VIDEO_TS/VIDEO_TS.BUP");
        //printf("ifoFileName: %s\n", ifoFileName);
        if (stat(ifoFileName, &statInfo) != 0 || stat(bupFileName, &statInfo) != 0)
        {
            strcpy(ifoFileName, argv[file]);
            if (ifoFileName[strlen(ifoFileName)-1] != '/')
                strcat(ifoFileName, "/");
            strcpy(bupFileName, ifoFileName);
            strcat(ifoFileName, "VIDEO_TS.IFO");
            strcat(bupFileName, "VIDEO_TS.BUP");
            if (stat(ifoFileName, &statInfo) != 0 || stat(bupFileName, &statInfo) != 0)
                return usage();
        }
        
        tsFile = fopen(bupFileName, "rb");
        if (tsFile)
        {
            fseek(tsFile, 0x4f4, SEEK_SET);
            numBytes = fread(bupBuffer, 1, sizeof(bupBuffer), tsFile);
            ///printf("%d bytes read from %s\n", numBytes, bupFileName);
            fclose(tsFile);
        }
        else
        {
            perror("Could not open BUP file");
            return 2;
        }
        
        tsFile = fopen(ifoFileName, "r+b");
        if (tsFile)
        {
            fseek(tsFile, 0x4f4, SEEK_SET);
            numBytes = fread(ifoBuffer, 1, sizeof(ifoBuffer), tsFile);
            ///printf("%d bytes read from %s\n", numBytes, ifoFileName);
            if (memcmp(ifoBuffer, bupBuffer, sizeof(ifoBuffer)) == 0)
            {
                fb = 0;
                if (memcmp(ifoBuffer, fixBuffer[0], sizeof(ifoBuffer)) == 0) // special case when original == first fix
                    ++fb;
            }
            else
                for (fb = 0; fb < NumBuffers; fb++)
                    if (memcmp(ifoBuffer, fixBuffer[fb], sizeof(ifoBuffer)) == 0)
                    {
                        ++fb; // write next
                        break;
                    }
            fseek(tsFile, 0x4f4, SEEK_SET);
            if (fb < NumBuffers)
                numBytes = fwrite(fixBuffer[fb], 1, sizeof(fixBuffer[fb]), tsFile);
            else // restore original from BUP
                numBytes = fwrite(bupBuffer, 1, sizeof(bupBuffer), tsFile);
            ///printf("%d bytes written from %s\n", numBytes, ifoFileName);
            fclose(tsFile);
        }
        else
        {
            perror("Could not open IFO file");
            return 3;
        }
    }
    return 0;
}
